angular
	.module( "beamng.apps" )
	.directive( "evBattery", [ "StreamsManager", function ( StreamsManager ) {
		return {
			template:
				"<object class='bngApp' style='width:100%; height:100%; pointer-events: none' type='image/svg+xml' data='modules/apps/EvBatteryMonitor/ev-battery-monitor.svg?t=" + Date.now() + "'/>",
			replace: true,
			restrict: "EA",
			link: function ( scope, element, attrs ) {
				function clamp( val, min, max ) {
					return Math.max( Math.min( val, max ), min );
				}

				function maprange( val, srcMin, srcMax, dstMin, dstMax, clampOutput ) {
					if ( clampOutput !== false ) clampOutput = true;
					if ( typeof dstMin !== "number" ) dstMin = 0.0;
					if ( typeof dstMax !== "number" ) dstMax = 1.0;

					var normalized = ( val - srcMin ) / ( srcMax - srcMin ); // 0.0 to 1.0 when inside[ srcMin, srcMax ]

					var dstRange = dstMax - dstMin;
					var output = dstRange * normalized + dstMin;

					if ( clampOutput ) {
						var cMin = Math.min( dstMin, dstMax );
						var cMax = Math.max( dstMin, dstMax );

						output = clamp( output, cMin, cMax );
					}

					return output;
				}

				StreamsManager.add( [ "hybrid" ] );

				scope.$on( "$destroy", function () {
					StreamsManager.remove( [ "hybrid" ] );
				} );

				element.on( "load", function () {
					var svg = element[ 0 ].contentDocument;
					var batteryLevelGauge = {
						bar: svg.getElementById( "fill-bat" ),
						val: svg.getElementById( "val-bat" ),
						height: svg.getElementById( "fill-bat" ).getAttribute( "height" ),
						factor: svg.getElementById( "fill-bat" ).getAttribute( "height" ),
						y: +svg.getElementById( "fill-bat" ).getAttribute( "y" )
					};
					var powerDrawGauge = {
						val: svg.getElementById( "val-kw" ),
						pos: {
							bar: svg.getElementById( "fill-kw-pos" ),
							height: svg.getElementById( "fill-kw-pos" ).getAttribute( "height" ),
							factor: svg.getElementById( "fill-kw-pos" ).getAttribute( "height" ),
							y: +svg.getElementById( "fill-kw-pos" ).getAttribute( "y" )
						},
						neg: {
							bar: svg.getElementById( "fill-kw-neg" ),
							factor: svg.getElementById( "fill-kw-neg" ).getAttribute( "height" ),
						}
					}

					scope.$on( "streamsUpdate", function ( event, streams ) {
						var stateOfCharge = 0,
							minStateOfCharge = 0,
							maxStateOfCharge = 1,
							powerDraw = 0,
							maxPowerDraw = 1;
						
						if ( streams != null && streams.hybrid != null ) {
							stateOfCharge = isNaN(streams.hybrid.soc) ? 0.0 : streams.hybrid.soc;
							minStateOfCharge = isNaN(streams.hybrid.minSoc) ? 0.0 : streams.hybrid.minSoc;
							maxStateOfCharge = isNaN(streams.hybrid.maxSoc) ? 1.0 : streams.hybrid.maxSoc;
							powerDraw = isNaN(streams.hybrid.kw) ? 0.0 : streams.hybrid.kw;
							maxPowerDraw = isNaN(streams.hybrid.maxKw) ? 0.0 : streams.hybrid.maxKw;
						}
						
						batteryLevelGauge.val.innerHTML = `${Math.round( stateOfCharge * 100 )}%`;
						powerDrawGauge.val.innerHTML = `${Math.round( powerDraw )} kW`;

						var normalizedStateOfCharge = maprange( stateOfCharge, minStateOfCharge, maxStateOfCharge );
						var batteryLevelHeight = normalizedStateOfCharge * batteryLevelGauge.factor;

						batteryLevelGauge.bar.setAttribute( "height", batteryLevelHeight );
						batteryLevelGauge.bar.setAttribute( "y", batteryLevelGauge.y + ( batteryLevelGauge.height - batteryLevelHeight ) );

						var powerDrawPosHeight = 0;
						var powerDrawNegHeight = 0;

						if ( maxPowerDraw === 0 ) {
							powerDrawPosHeight = powerDrawNegHeight = 0;
						} else {
							var powerDrawPosNormalized = isNaN( powerDraw ) ? 0 : clamp( powerDraw / maxPowerDraw, 0, 1 );
							var powerDrawNegNormalized = isNaN( powerDraw ) ? 0 : clamp( powerDraw / maxPowerDraw, -1, 0 );

							powerDrawPosHeight = Math.abs( powerDrawPosNormalized ) * powerDrawGauge.pos.factor;
							powerDrawNegHeight = Math.abs( powerDrawNegNormalized ) * powerDrawGauge.neg.factor;
						}

						powerDrawGauge.pos.bar.setAttribute( "height", powerDrawPosHeight );
						powerDrawGauge.pos.bar.setAttribute( "y", powerDrawGauge.pos.y + ( powerDrawGauge.pos.height - powerDrawPosHeight ) );
						powerDrawGauge.neg.bar.setAttribute( "height", powerDrawNegHeight );
					} );
				} );
			}
		};
	} ] );
